'=========================================================================
' Prozeduren fuer Grafik (32 Bit, 8x16 Zeichensatz), hauptschlich Diagramme
' Graphik.bas
' Uebernommen aus QuickBASIC Januar 2006
' Letzte Ueberarbeitung 18.12.13
'=========================================================================

'Die folgenden Diagramme werden mitsamt der Beschriftung in ein mit View definiertes
'Darstellungsfeld eingepasst. Falls kein solches definiert wurde, wird das gesamte 
'Grafikfenster verwendet. Die Diagramme erzeugen jedoch anschliessend mit View
'ein neues (kleineres) Darstellungsfeld! 


#Include Once "Common.bi"
#Include Once "fbgfx.bi"
#Include Once "Mathe.bas"


Sub Initgfx(Titel As String, Breite As Integer = 640, Hoehe As Integer = 480)
  'Initialisiert den Grafikbildschirm mit Titel und den angegebenen Abmessungen, 
  'Farbtiefe 32 Bit, Seitenzahl = 2, 8x16 Zeichensatz.
  
  Screenres(Breite, Hoehe, 32, 2)
  Windowtitle Titel
  Width Breite \ 8, Hoehe \ 16
  
End Sub


Function Outputwidth() As Integer
  'Breite des mit View gesetzten Darstellungsfeldes
  
  Dim As Integer x0, y0, x1, y1 
  
  Using FB
  Screencontrol GET_VIEWPORT, x0, y0, x1, y1
  Return x1 - x0
  
End Function


Function Outputheight() As Integer
  'Hhe des mit View gesetzten Darstellungsfeldes
  
  Dim As Integer x0, y0, x1, y1 
  
  Using FB
  Screencontrol GET_VIEWPORT, x0, y0, x1, y1
  Return y1 - y0
  
End Function


Function TextWidth(s As String = Space(1)) As Double
  'Textbreite eines String s in Pixeln bzw. in x-Sichfensterkoordinaten
  
  Dim As Double x1, x2, TWidth
  
  TWidth = 8 * Len(s) 
  'Falls ein Koordinatensystem mit Window() gesetzt wurde, wird die Textbreite in
  'mathematischen x-Werten berechnet:
  x1 = Pmap(0, 2)
  x2 = Pmap(TWidth, 2)
  TWidth = x2 - x1
  Return TWidth
  
End Function


Function TextHeight(s As String = Space(1)) As Double
  'Texthoehe eines String s in Pixeln bzw. in y-Sichfensterkoordinaten
  
  Dim As Double y1, y2, THeight
  
  THeight = 16 
  'Falls ein Koordinatensystem mit Window() gesetzt wurde, wird die Texthoehe in
  'mathematischen y-Werten berechnet:
  y1 = Pmap(0, 3)
  y2 = Pmap(THeight, 3)
  THeight = Abs(y2 - y1)
  Return THeight
  
End Function


Sub Drawrotatedstring(x As Integer, y As Integer, s As String, Farbe As Integer, Winkel As Double)
  'String s mit der Vordergrundfarbe zeichnen wie mit Draw String, jedoch um den Punkt (x, y)
  'gedreht (Winkel in Grad)
  
  Dim As Integer xrot, yrot, i, j, HFarbe
  Dim buffer As Any Pointer
  Dim As Double phi = -pi * Winkel / 180
  
  'Eine Hintergrundfarbe fr den Buffer auswrfeln:
  Do
    HFarbe = Rgb(255 * Rnd, 255 * Rnd, 255 * Rnd) 
  Loop Until HFarbe <> Farbe
  
  buffer = Imagecreate(TextWidth(s), TextHeight(s), HFarbe)
  Draw String buffer, (0, 0), s, Farbe 
  For i = 0 To TextWidth(s)
    For j = 0 To TextHeight(s)
      If Point(i, j, buffer) = Farbe Then
        xrot = i * Cos(phi) - j * Sin(phi)
        yrot = i * Sin(phi) + j * Cos(phi)
        Pset (x + xrot, y + yrot), Farbe
      End If
    Next
  Next
  
  Imagedestroy(buffer)
  
End Sub


Sub Koordinaten(xmin As Double, ymin As Double, xmax As Double, ymax As Double, _
  nx As Integer, ny As Integer, hor As String, vert As String, Text As String)
  ' Koordinatenfeld mit linearen Achsenteilungen.
  ' Parameter:
  ' - xmin ... ymax definieren die mathematischen Abmessungen des Koordinatenfeldes.
  ' - nx, ny = Zahl der Achsenabschnitte auf der x- bzw. y-Achse
  ' - hor, vert = Beschriftung fuer x-Achse bzw. y-Achse
  ' - Text = Bildunterschrift
  
  Dim As Integer i, s(2), maxs, xlo, ylo, xru, yru, x0, y0, x1, y1
  Dim As Double ax, ay, x, y
  Dim As String xmins, ymins, xmaxs, ymaxs
  
  xmins = Str(Csng(xmin))
  ymins = Str(Csng(ymin)) + Space(1)
  xmaxs = Str(Csng(xmax))
  ymaxs = Str(Csng(ymax)) + Space(1)
  vert = vert + Space(1)
  
  'Lngsten Wert bei der Beschriftung der vertikalen Achse ermitteln:
  s(0) = Len(ymins)
  s(1) = Len(ymaxs)
  s(2) = 0.5 * Len(xmins)
  maxs = MaxInt(s()) + 1
  
  'Physikalische Koordinaten der linken oberen und unteren rechten Ecke des Diagrammfeldes:
  xlo = maxs * TextWidth()
  ylo = TextHeight()
  xru = Outputwidth() - 0.5 * TextWidth(xmaxs + Space(2))
  yru = Outputheight() - 4 * TextHeight()
  
  'Beschriftung vertikale Achse:
  Draw String (xlo - TextWidth(ymaxs), ylo - 0.5 * TextHeight()), ymaxs
  Draw String (xlo - TextWidth(ymins), yru - 0.5 * TextHeight()), ymins
  If Len(vert) < maxs Then
    Draw String (xlo - Textwidth(vert), (ylo + yru) / 2 - 0.5 * TextHeight()), vert
  Else
    Drawrotatedstring(xlo - TextWidth() - TextHeight(), (ylo + yru) / 2 + 0.5 * TextWidth(vert), _
    vert, Color, 90)
  End If
  
  'Beschriftung horizontale Achse:
  Draw String (xlo - 0.5 * TextWidth(xmins), yru + 0.5 * TextHeight()), xmins
  Draw String (xru - 0.5 * TextWidth(xmaxs), yru + 0.5 * TextHeight()), xmaxs
  Draw String ((xlo + xru) / 2 - 0.5 * TextWidth(hor), yru + 0.5 * TextHeight()), hor
  
  'Bildunterschrift:
  Draw String ((xlo + xru) / 2 - 0.5 * TextWidth(Text), yru + 1.5 * TextHeight()), Text
  
  'Fenster fuer Koordinatensystem in Sichtkoordinaten definieren:
  Using FB
  Screencontrol GET_VIEWPORT, x0, y0, x1, y1
  View (x0 + xlo, y0 + ylo)-(x0 + xru, y0 + yru),,Color
  Window (xmin, ymin)-(xmax, ymax)
  
  'Teilung horizontale Achse:
  ax = (xmax - xmin) / nx
  For i = 0 To nx
    x = xmin + i * ax
    Line (x, ymin)-(x, ymax)
  Next i
  
  'Teilung vertikale Achse:
  ay = (ymax - ymin) / ny
  For i = 0 To ny
    y = ymin + i * ay: Line (xmin, y)-(xmax, y)
  Next i
  
End Sub


Sub Zeichnekurve (x() As Double, y() As Double, Farbe As Integer)
  ' Kurve durch die Punkte x(0), y(0) bis x(n), y(n) als Polygonzug in der
  ' angegebenen Farbe, z.B. in das mit der SUB Koordinaten gezeichnete
  ' Koordinatensystem.
  
  Dim As Integer n, i
  
  n = Ubound(x)
  For i = 1 To n
    Line (x(i - 1), y(i - 1))-(x(i), y(i)), Farbe
  Next i
  
End Sub


Sub Zeichnefkt(Fkt As Function (x As Double) As Double, xmin As Double, _
  xmax As Double, Farbe As Integer)
  ' Zeichnet ein Funktion fkt(x) im Wertebereich xmin bis xmax in der angegebenen
  ' Farbe, z.B. in das mit der SUB Koordinaten gezeichnete Koordinatensystem.
  
  Dim As Double dx, x
  
  dx = (xmax - xmin) / 300
  For x = xmin To xmax - dx Step dx
    Line (x, Fkt(x)) - (x + dx, Fkt(x + dx)), Farbe
  Next x
  
End Sub


Sub Balkendiagramm(x() As String, y() As Double, ymin As Double, ymax As Double,_
  ny As Integer, hor As String, vert As String, Text As String, Farbe As Integer)
  ' - x(0) bis x(n) = Benennung der Balken
  ' - y(0) bis y(n) = Wert der Balken
  ' - n + 1         = Anzahl der Balken
  ' - ymin, ymax    = Abmessung vertikale Achse (ymin muss <= 0 sein!)
  ' - ny            = Anzahl der Achsenabschnitte auf der vertikalen Achse
  ' - hor           = Beschriftung horizontale Achse
  ' - vert          = Beschriftung vertikale Achse (max 13 Zeichen)
  ' - Text          = Bildunterschrift
  ' - Farbe         = Farbe
  
  Dim As Integer n = Ubound(x), i, s(1), maxs, xlo, ylo, xru, yru, x0, y0, x1, y1
  Dim As Double xmin, xmax, ay, yWert, Breite
  Dim As String ymins, ymaxs
  
  ymins = Str(Csng(ymin)) + Space(1)
  ymaxs = Str(Csng(ymax)) + Space(1)
  vert = vert + Space(1)
  
  'Lngsten Wert bei der Beschriftung der vertikalen Achse ermitteln:
  s(0) = Len(ymins)
  s(1) = Len(ymaxs)
  maxs = MaxInt(s()) + 1
  
  'Physikalische Koordinaten der linken oberen und unteren rechten Ecke des Diagrammfeldes:
  xlo = maxs * TextWidth()
  ylo = TextHeight()
  xru = OutputWidth() - 5 * TextWidth()
  yru = OutputHeight() - 4 * TextHeight()
  
  'Beschriftung vertikale Achse:
  Draw String (xlo - TextWidth(ymaxs), ylo - 0.5 * TextHeight()), ymaxs
  Draw String (xlo - TextWidth(ymins), yru - 0.5 * TextHeight()), ymins
  If Len(vert) < maxs Then
    Draw String (xlo - Textwidth(vert), (ylo + yru) / 2 - 0.5 * TextHeight()), vert
  Else
    DrawRotatedString(xlo - TextWidth() - TextHeight(), (ylo + yru) / 2 + 0.5 * TextWidth(vert), _
    vert, Color, 90)
  End If
  
  'Beschriftung horizontale Achse:
  For i = 0 To n
    Draw String (xlo + (xru - xlo) * (i + 0.5) / (n + 1) - 0.5 * TextWidth(x(i)), yru + 0.5 * TextHeight()), x(i)
  Next i
  Draw String ((xlo + xru) / 2 - 0.5 * TextWidth(hor), yru + 1.5 * TextHeight()), hor
  
  'Bildunterschrift:
  Draw String ((xlo + xru) / 2 - 0.5 * TextWidth(Text), yru + 2.5 * TextHeight()), Text
  
  'Fenster fuer Graphik in Sichtkoordinaten definieren:
  Using FB
  Screencontrol GET_VIEWPORT, x0, y0, x1, y1
  View (x0 + xlo, y0 + ylo)-(x0 + xru, y0 + yru),,Color
  xmin = -0.5: xmax = n + 0.5
  Window (xmin, ymin)-(xmax, ymax)
  
  'Teilung vertikale Achse:
  ay = (ymax - ymin) / ny
  For i = 0 To ny
    yWert = ymin + i * ay
    Line (xmin, yWert)-(xmax, yWert)
  Next i
  
  'Balken zeichnen:
  For i = 0 To n
    Line(i - 0.4, 0) - (i + 0.4, y(i)), Farbe, BF
  Next i
  
End Sub


Sub Kreisdiagramm(x() As String, y() As Double, Farbe() As Integer, Text As String)
  ' - x(0) bis x(n) = Bezeichnung der Kreissegmente
  ' - y(0) bis y(n) = Wert der Kreissegmente
  ' - Farbe(0) bis Farbe(n) = Farben der Kreissegmente 
  ' - n + 1         = Anzahl der Kreissegmente
  ' - Text          = Bildunterschrift
  
  Dim As Integer n = Ubound(x), i, x0, y0, r
  Dim As Double Alpha(n), sum, xp, yp, beta
  Dim As String ys
    
  'Kreismittelpunkt und -radius:
  x0 = 0.5 * OutputWidth()
  y0 = 0.45 * OutputHeight()
  r = 0.35 * OutputHeight()
  
  'Diagramm berechnen und segmentweise zeichnen (alpha(0) bis alpha(n) sind
  'die zu y(0) bis y(n) zugehoerigen Winkel im Kreis:
  sum = 0
  For i = 0 To n
    sum = sum + y(i)
  Next i
  'Die Winkel alpha(i) zaehlen ab pi/2 (oben) im Uhrzeigersinn, also entgegen-
  'gesetzt zum mathematischen Drehsinn:
  Alpha(0) = pi / 2 - 2 * pi * y(0) / sum
  For i = 1 To n
    Alpha(i) = Alpha(i - 1) - 2 * pi * y(i) / sum
  Next i
  Circle (x0, y0), r
  Line (x0, y0) - (x0, y0 - r)
  For i = 0 To n
    Line (x0, y0) - (x0 + r * Cos(Alpha(i)), y0 - r * Sin(Alpha(i)))
  Next i
  
  'Diagramm-Segmente faerben und beschriften:
  For i = 0 To n
    If i = 0 Then beta = (pi / 2 + Alpha(0)) / 2 _
    Else beta = (Alpha(i) + Alpha(i - 1)) / 2
    xp = x0 + 0.5 * r * Cos(beta)
    yp = y0 - 0.5 * r * Sin(beta)
    Paint (xp, yp), Farbe(i), Loword(Color)    
    xp = x0 + 1.1 * r * Cos(beta)
    yp = y0 - 1.1 * r * Sin(beta)
    ys = Str(Csng(y(i)))
    If beta < -pi / 2 Then
      Draw String (xp - TextWidth(ys), yp - 0.5 * TextHeight()), ys
    Else
      Draw String (xp, yp - 0.5 * TextHeight()), ys
    End If
  Next i
  
  'Bildunterschrift:
  Draw String (x0 - 0.5 * TextWidth(Text), y0 + 1.3 * r - TextHeight()), Text
  
End Sub


Function RGBWert(z As Double, Stufen As Integer = 0) As Uinteger
  ' Berechnet eine RGB-Farbnummer abhaengig vom Wert z in der angegebenen Zahl von Stufen,
  ' z.B. fuer Kontourplots. Wertebereich fuer z = 0 bis 1, Farbskala blau - blaugruen - gruen - gelb - rot)
  ' Stufen = 0: keine Stufung, d.h. Wertebereich ist kontinuierlich
  
  Dim As Double Wert
  
  If Stufen <> 0 Then
    z = Cint(Stufen * z) / Stufen
  End If
  
  If z < 0 Then
    Wert = Rgb(0, 0, 250)
  Elseif z < 0.25 Then
    Wert = Rgb(0, 1000 * z, 250)
  Elseif z < 0.5 Then 
    Wert = Rgb(0, 250, 250 - 1000 * (z - 0.25))
  Elseif z < 0.75 Then
    Wert = Rgb(1000 * (z - 0.5), 250, 0)
  Elseif z < 1 Then
    Wert = Rgb(250, 250 - 1000 * (z - 0.75), 0)
  Else
    Wert = Rgb(250, 0, 0)
  End If
  
  Return Wert
  
End Function


Sub PlotFkt(fkt As Function (x As Double, y As Double) As Double, _ 
  xmin As Double, ymin As Double, xmax As Double, ymax As Double, _
  Stufen As Integer = 20, Koord As Integer = 0)
  ' Zeichnet eine Funktion fkt(x, y) als "Hoehenkarte" (Graphik mit 32 Bit Farbtiefe).
  ' Parameter:
  ' - xmin, xmax, ymin, ymax = Wertebereich von x und y
  ' Optionale Parameter:
  ' - Stufen = Anzahl der Farbstufen (siehe Function RGBWert)
  ' - Koord = 0: ganzes Graphikfenster als Diagrammflaeche
  '   Koord <> 0: Diagrammflaeche wie bei Sub Koordinaten. Mit der SUB Koordinaten kann dann
  '   vorher ein Koordinatensystem gezeichnet werden (die Sub Koordinaten enthlt den entsprechenden
  '   View-Befehl fr das Sichtfenster).
  ' Benoetigte Prozedur: FUNCTION fkt (x, y) (im Hauptmodul zu definieren)
  ' Aufruf der SUB im Hauptmodul: PlotFkt(@fkt, (sonstige Parameter))
  
  Dim As Double x, y, dx, dy, z, zmin, zmax
  
  Window (xmin, ymin) - (xmax, ymax)
  
  'Pixelgenaue Intervalle fuer x und y:
  dx = (xmax - xmin) / (OutputWidth() + 1)
  dy = (ymax - ymin) / (OutputHeight() + 1)
  
  'Minimalen und maximalen Wert von z suchen:
  zmin = 0
  zmax = 0
  For x = xmin To xmax
    For y = ymin To ymax
      z = fkt(x, y)
      If z < zmin Then zmin = z
      If z > zmax Then zmax = z
    Next y
  Next x
  
  'Zeichnen:
  For x = xmin To xmax Step dx
    For y = ymax To ymin Step -dy
      z = fkt(x, y)
      Pset(x, y), RGBWert((z - zmin) / (zmax - zmin), Stufen)
    Next y
  Next x  
  
End Sub
