'=============================================================================
' Rechnen mit komplexen Zahlen 
' KOMPLEX.BAS
' Erstellt Maerz 2006
' Letzte Ueberarbeitung 18.12.13
'=============================================================================


#Include Once "Common.bi"


'Komplexe Zahl:
TYPE COMPLEX
  Re AS DOUBLE
  Im AS DOUBLE
END Type


Function CAbs(z AS COMPLEX) AS DOUBLE
' Betrag einer komplexen Zahl
  CAbs = SQR(z.Re^2 + z.Im^2)
END FUNCTION


FUNCTION CArg(z AS COMPLEX) AS DOUBLE
' Argument (Winkel) einer komplexen Zahl im Bogenmass

  DIM AS DOUBLE phi
  
  phi = ATAN2(z.Im, z.Re)
' ATAN2 kann zwischen -pi und + pi liegen. Korrektur fuer -pi < phi < 0:
  IF phi < 0 THEN CArg = 2 * pi + phi ELSE CArg = phi
  
END FUNCTION


FUNCTION CTransform(r AS DOUBLE, phi AS DOUBLE) AS COMPLEX
' Komplexe Zahl mit Betrag r und Argument phi (Bogenmass) (entspricht einer
' Transformation von Polarkoordinaten in kartesische Koordinaten)

  DIM AS COMPLEX c
  
  c.Re = r * COS(phi)
  c.Im = r * SIN(phi)
  CTransform = c
  
END FUNCTION


FUNCTION CAdd(a AS COMPLEX, b AS COMPLEX) AS COMPLEX
' Addition zweier komplexer Zahlen: c = a + b

  DIM AS COMPLEX c
  
  c.Re = a.Re + b.Re
  c.Im = a.Im + b.Im
  CAdd = c
  
END FUNCTION


FUNCTION CSub(a AS COMPLEX, b AS COMPLEX) AS COMPLEX
' Subtraktion zweier komplexer Zahlen: c = a - b

  DIM AS COMPLEX c
  
  c.Re = a.Re - b.Re
  c.Im = a.Im - b.Im
  CSub = c
  
END FUNCTION


FUNCTION CMult(a AS COMPLEX, b AS COMPLEX) AS COMPLEX
' Multiplikation zweier komplexer Zahlen: c = a * b

  DIM AS DOUBLE r, phi
  
  r = CAbs(a) * CAbs(b)
  phi = CArg(a) + CArg(b)
  CMult = CTransform(r, phi)
  
END FUNCTION


FUNCTION CDiv(a AS COMPLEX, b AS COMPLEX) AS COMPLEX
' Division zweier komplexer Zahlen: c = a / b

  DIM AS DOUBLE r, phi
  
  r = CAbs(a) / CAbs(b)
  phi = CArg(a) - CArg(b)
  CDiv = CTransform(r, phi)
  
END FUNCTION


FUNCTION CPotenz(a AS COMPLEX, n AS DOUBLE) AS COMPLEX 
' n-te Potenz einer komplexen Zahl: b = a^n
' Fuer n = 1/m (m natuerliche Zahl) entspricht das Ergebnis dem Hauptwert 
' der m-ten Wurzel aus z. Insgesamt gibt es m Wurzeln vom gleichen Betrag
' mit den Argumenten CArg(a)/m + 2*pi*k/m, k = 0 bis m-1. Loesungen fuer m = 2
' siehe SUB CSqr.

  DIM AS DOUBLE r, phi
  
  r = CAbs(a)^n
  phi = CArg(a) * n
  CPotenz = CTransform(r, phi)
  
END FUNCTION


SUB CSqr(a AS COMPLEX, BYREF b1 AS COMPLEX, BYREF b2 AS COMPLEX)
' Quadratwurzeln aus einer komplexen Zahl: b1 und b2 = a^0.5

  DIM AS DOUBLE r, phi1, phi2
    
  r = SQR(CAbs(a))
  phi1 = CArg(a)/2
  phi2 = CArg(a)/2 + pi
  b1 = CTransform(r, phi1)
  b2 = CTransform(r, phi2)
  
END SUB


