'=========================================================================
' Prozeduren fuer Mathematik
' Mathe.bas
' Uebernommen aus QuickBASIC Januar 2006
' Letzte Ueberarbeitung 18.12.13
'=========================================================================


#Include Once "Common.bi"
#Include Once "Komplex.bas"


Function Fak(x As Integer) As Double
  ' Fakultaet von x: x! = 1 * 2 * 3 * ... * x
  
  Dim As Integer i
  Dim As Double y
  
  If x = 0 Then
    y = 1
  Else
    y = 1
    For i = 1 To x
      y = y * i
    Next i
  End If
  Fak = y
  
End Function


Function Binkoeff(n As Integer, k As Integer) As Integer
  ' Binominaloeffizient "n ber k"
  
  Binkoeff = Fak(n) / (Fak(n - k) * Fak(k))
  
End Function


Function Integral(x() As Double, y() As Double) As Double
  ' Bestimmtes Integral ueber y(x) nach der Simpsonschen Regel,
  ' x und y = Felder mit den Indizes 0 bis n,
  ' Werte von x muessen quidistant sein, n geradzahlig.
  
  Dim As Double Wert, h
  Dim As Integer n, i
  
  n = Ubound(x)
  Wert = 0
  h = x(1) - x(0)
  For i = 2 To n Step 2
    Wert = Wert + h * (y(i - 2) + 4 * y(i - 1) + y(i)) / 3
  Next i
  Integral = Wert
  
End Function


Function LOGx(x As Double, b As Double) As Double
  ' Logarithmus von x zur Basis b
  ' Beispiel: LOGx(100, 10) = 2 
  
  LOGx = Log(x) / Log(b)
  
End Function


Function MaxDbl (a() As Double) As Double
  ' Maximaler Wert in einem Datenfeld a(0) bis a(n) vom Typ DOUBLE
  
  Dim As Integer n, i
  n = Ubound(a)
  Dim Wert As Double
  
  Wert = a(0)
  For i = 1 To n
    If a(i) > Wert Then Wert = a(i)
  Next i
  Return Wert
  
End Function


Function MaxInt (k() As Integer) As Integer
  ' Maximaler Wert in einem Datenfeld k(0) bis k(n) vom Typ INTEGER
  
  Dim As Integer n, i
  n = Ubound(k)
  Dim Wert As Double
  
  Wert = k(0)
  For i = 1 To n
    If k(i) > Wert Then Wert = k(i)
  Next i
  Return Wert
  
End Function


Function MinDbl (a() As Double) As Double
  ' Minimaler Wert in einem Datenfeld a(0) bis a(n) vom Typ DOUBLE
  
  Dim As Integer n, i
  n = Ubound(a)
  Dim Wert As Double
  
  Wert = a(0)
  For i = 1 To n
    If a(i) < Wert Then Wert = a(i)
  Next i
  Return Wert
  
End Function


Function MinInt (k() As Integer) As Integer
  ' Minimaler Wert in einem Datenfeld k(0) bis k(n) vom Typ INTEGER
  
  Dim As Integer n, i
  n = Ubound(k)
  Dim Wert As Double
  
  Wert = k(0)
  For i = 1 To n
    If k(i) < Wert Then Wert = k(i)
  Next i
  MinInt = Wert
  
End Function


Sub LinGlSys(a() As Double, x() As Double, b() As Double)
  ' Loesung des linearen Gleichungssystems
  ' a(1,1)*x(1) + a(1,2)*x(2) + ... + a(1,n)*x(n) = b(1)
  ' a(2,1)*x(1) + a(2,2)*x(2) + ... + a(2,n)*x(n) = b(2)
  ' ................
  ' a(n,1)*x(1) + a(n,2)*x(2) + ... + a(n,n)*x(n) = b(n)
  ' nach dem Gausschen Eliminationsverfahren.
  ' Bedingungen:
  ' - n > 1
  ' - Determinante von a <> 0
  ' - Alle Diagonalelemente von a <> 0
  ' Siehe Bronstein/Semendjajew, Seite 735 f.
  ' Anmerkung: Das Determinantenverfahren erwies sich fuer grosse n als viel zu 
  ' langsam!
  
  Dim As Integer n, i, j, k
  n = Ubound(a, 1)
  Dim As Double Matrix(n, n), Spalte(n), q(n), sum
  
  'a() und b() retten:
  For i = 1 To n
    Spalte(i) = b(i)
    For j = 1 To n
      Matrix(i, j) = a(i, j)
    Next j
  Next i
  
  'Transformation von Matrix auf Dreiecksgestalt:
  k = 1
  Do
    For i = k + 1 To n
      q(i) = -Matrix(i, k) / Matrix(k, k)
      For j = k To n
        If j = k Then
          Matrix(i, j) = 0
        Else
          Matrix(i, j) = Matrix(i, j) + q(i) * Matrix(k, j)
        End If
      Next j
      Spalte(i) = Spalte(i) + q(i) * Spalte(k)
    Next i
    k = k + 1
  Loop While k < n
  
  'Berechnung des Loesungsvektors x:
  x(n) = Spalte(n) / Matrix(n, n)
  For i = n - 1 To 1 Step -1
    sum = 0
    For j = 1 To n - i
      sum = sum + Matrix(i, i + j) * x(i + j)
    Next j
    x(i) = (Spalte(i) - sum) / Matrix(i, i)
  Next i
  
End Sub


Function Polynom(a() As Double, x As Double) As Double
  ' Berechnung des Polynomwertes a(0) + a(1) * x + a(2) * x^2 + ... + a(n) * x^n,
  ' a(0) bis a(n) = Polynomkoeffizienten
  
  Dim As Integer k, n = Ubound(a)
  Dim As Double sum
  
  sum = 0
  For k = 0 To n
    sum = sum + a(k) * x ^ k
  Next k
  Polynom = sum
  
End Function


Sub PolyAbl(a() As Double, b() As Double)
  ' Berechnet die Koeffizienten b(0) bis b(n - 1) der Ableitung des Polynoms
  ' a(0) + a(1) * x + a(2) * x^2 + ... + a(n) * x^n
  
  Dim As Integer k, n = Ubound(a)
  
  For k = 0 To n - 1
    b(k) = (k + 1) * a(k + 1)
  Next k
  
End Sub


Sub Interpol(x() As Double, y() As Double, a() As Double)
  ' Berechnet durch die Punkte x(0), y(0) bis x(n), y(n) die Koeffizienten a(0)
  ' bis a(n) des Interpolationsplynoms durch Loesen des linearen Gleichungssystems
  ' a(0) + a(1) * x(0) + a(2) * x(0)^2 + ... + a(n) * x(0)^n = y(0)
  ' a(0) + a(1) * x(1) + a(2) * x(1)^2 + ... + a(n) * x(1)^n = y(1)
  ' ...
  ' a(0) + a(1) * x(n) + a(2) * x(n)^2 + ... + a(n) * x(n)^n = y(n)
  
  Dim As Integer i, j, n = Ubound(x)
  Dim As Double KoeffMatrix(1 To n + 1, 1 To n + 1), z(1 To n + 1), b(1 To n + 1)
  
  For i = 1 To n + 1
    b(i) = y(i - 1)
    For j = 1 To n + 1
      KoeffMatrix(i, j) = x(i - 1) ^ (j - 1)
    Next j
  Next i
  
  LinGlSys(KoeffMatrix(), z(), b())
  
  For i = 0 To n
    a(i) = z(i + 1)
  Next i
  
End Sub


Sub QuadGl(a As Double, b As Double, c As Double, Byref x1 As COMPLEX, _
  Byref x2 As COMPLEX)
  ' Loesung der quadratischen Gleichung a * x^2 + b * x + c = 0,
  ' Loesungen: x1 und x2 (komplex) 
  
  Dim D As Double
  
  D = b^2 - 4 * a * c   'D = Diskriminante
  If D < 0 Then
    x1.Re = -b / (2 * a)
    x1.Im = Sqr(-D) / (2 * a)
    x2.Re = -b / (2 * a)
    x2.Im = -Sqr(-D) / (2 * a)
  Else
    x1.Re = (-b + Sqr(D)) / (2 * a)
    x2.Re = (-b - Sqr(D)) / (2 * a)
    x1.Im = 0
    x2.Im = 0
  End If
  
End Sub


Sub KubGl(a As Double, b As Double, c As Double, d As Double, _
  Byref x1 As Double, Byref x2 As COMPLEX, Byref x3 As COMPLEX)
  ' Loesung der kubischen Gleichung a * x^3 + b * x^2 + c * x + d = 0,
  ' Loesungen: x1 (rell), x2 und x3 (komplex)
  
  Dim As Double r, s, t, p, q, pq, u1, v1, rho, phi, y1
  Dim As COMPLEX y2, y3
  
  'Normalform: x^3 + r * x^2 + s * x + t = 0
  r = b / a
  s = c / a
  t = d / a
  
  'Loesen der reduzierten kubischen Gleichung y^3 + p * y + q = 0:
  p = s - r^2 / 3
  q = 2 * r^3 / 27 - r * s / 3 + t
  pq = (q / 2)^2 + (p / 3)^3
  If pq >= 0 Then
    'Cardanische Formel: eine reelle und zwei konjugiert komplexe Loesungen
    u1 = (-q / 2 + Sqr(pq))
    If u1 < 0 Then u1 = (-u1) ^ (1 / 3) Else u1 = u1 ^(1 / 3)
    v1 = (-q / 2 - Sqr(pq))
    If v1 < 0 Then v1 = (-v1) ^ (1 / 3) Else v1 = v1 ^(1 / 3) 
    y1 = u1 + v1
    y2.Re = -(u1 + v1) / 2
    y2.Im = (u1 - v1) * Sqr(3) / 2
    y3.Re = y2.Re
    y3.Im = -y2.Im
  Else
    'Casus irreduzibilis: drei reelle Losungen
    rho = Sqr(-p^3 / 27)
    phi = Acos(-q / (2 * rho))
    y1 = 2 * rho^(1 / 3) * Cos(phi / 3)
    y2.Re = 2 * rho^(1 / 3) * Cos(phi / 3 + 2 * pi / 3)
    y2.Im = 0
    y3.Re = 2 * rho^(1 / 3) * Cos(phi / 3 + 4 * pi / 3)
    y3.Im = 0
  End If
  
  x1 = y1 - r / 3
  x2.Re = y2.Re - r / 3
  x2.Im = y2.Im
  x3.Re = y3.Re - r / 3
  x3.Im = y3.Im
  
End Sub


Sub RegulaFalsi(x() As Double, y() As Double, x0() As Double, _
  Byref Code As Integer)
  ' Nullstellen einer Funktion y(x) (Regula Falsi), Parameter:
  ' - y und x = Felder mit Indizes von 0 bis n, x aufsteigend
  ' - x0() = Array der gefundene Nullstellen (Index beginnend ab 0)
  ' - Code: Wenn keine Nullstellen gefunden wurden, wird Code = 1 gesetzt, sonst 0
  ' Methode: Lineare Interpolation zwischen den Stuetzwerten bei Vorzeichenwechsel
  
  Dim As Integer n, i, k
  
  n = Ubound(x)
  Code = 1
  Redim x0(0)
  k = -1
  For i = 0 To n - 1
    If y(i) = 0 Then
      'direkt gefundene Nullstellen:
      Code = 0
      k = k + 1
      Redim Preserve x0(k)
      x0(k) = x(i)
    Elseif Sgn(y(i + 1)) <> Sgn(y(i)) And y(i+1) <> 0 Then
      'Lineare Interpolation (Regula falsi):
      Code = 0
      k = k + 1
      Redim Preserve x0(k)      
      x0(k) = x(i) - y(i) * (x(i + 1) - x(i)) / (y(i + 1) - y(i))
    End If
  Next i
  
End Sub


Function SumDbl (a() As Double) As Double
  ' Summe der Feldelemente a(0) bis a(n) vom Typ DOUBLE
  
  Dim sum As Double
  Dim As Integer n, i
  
  n = Ubound(a)
  Sum = 0
  For i = 0 To n
    Sum = Sum + a(i)
  Next i
  Return Sum
  
End Function


Function SumInt(k() As Integer) As Integer
  ' Summe der Feldelemente k(0) bis k(n) vom Typ INTEGER
  
  Dim sum As Integer
  Dim As Integer n, i
  
  n = Ubound(k)
  Sum = 0
  For i = 0 To n
    Sum = Sum + k(i)
  Next i
  Return Sum
  
End Function




