'==============================================================================
' Mandel.bas
' Mandelbrot-Fraktal mit Zoom-Funktion
' Erstellt am 06.05.10
' Letzte Bearbeitung am 20.12.11
'==============================================================================


#INCLUDE "Inc\KOMPLEX.bas"
#INCLUDE "Inc\GRAPHIK.bas"
#INCLUDE "Inc\TUI.bas"


Dim Shared As Single amin, amax, bmin, bmax
Dim As Integer key


Sub Info

  Screen 0
  Color 0, 7
  Cls
  
  Zentriert(1, "Mandelbrot-Fraktal")
  Print
  Print "Das Mandelbrot-Fraktal beruht auf der Iteration z -> z^2 + c, aus der"
  Print "das Julia-Fraktal erzeugt wird. z und c sind komplexe Zahlen, mit"
  Print "c = a + i * b. Das Mandelbrot-Fraktal entsteht hieraus nach folgendem"
  Print "Verfahren: Fuer jedes Wertepaar a und b wird untersucht, ob die aus der"
  Print "Iteration erzeugte Folge der z-Werte konvergiert oder nicht. Hierbei"
  Print "wird c als Anfangswert fuer z hergenommen. Im Programm wird geprueft,"
  Print "nach wie vielen Iterationen der Betrag von z einen Mindestwert"
  Print "ueberschreitet (hier 100), wobei nach maximal 50 Iterationen"
  Print "abgebrochen wird. Die Zahl der Iterationen wird in der a-b-Ebene fuer"
  Print "jeden Wert von (a, b) als Farbwert eingetragen. In der berechneten"
  Print "Grafik kann mit Hilfe der Maus ein Ausschnitt zum Vergroessern"
  Print "ausgewaehlt werden (Zoom-Funktion)."
  Print
  Print "Literatur: Hans Lauwerier, Fraktale."
  Locate 25, 1
    
End Sub


Sub Mandel
  
  Dim As Single a, b, da, db
  Dim As COMPLEX z, c, z1, z2
  Dim As Integer i, j, count
  
  Screenres 640, 480
  Windowtitle "Mandelbrot-Fraktal"
  Width 80, 30
  View Print 29 To 30
  View (0, 0) - (640, 440)
  Window (amin, bmin)-(amax, bmax)
  da = (amax - amin) / 640
  db = (bmax - bmin) / 480
  For i = 1 To 640
    a = amin + i * da
    For j = 1 To 480
      b = bmin + j * db
      c.Re = a
      c.Im = b
      z.Re = a
      z.Im = b
      count = 0
      Do
        z = CAdd(CPotenz(z, 2), c)
        count = count + 1
      Loop Until CAbs(z) > 100 Or count > 50
      If count > 50 Then Pset (a, b), 0 Else Pset (a, b), 32 + count
    Next j
  Next i
  
End Sub
        
        
Sub Zoom
  
  Dim As Integer xp, yp, Taste
  
  Print "Linke obere Ecke des gewuenschten Ausschnittes anklicken!"
  Do
    Sleep 1
    Getmouse xp, yp,,Taste
  Loop Until Bit(Taste, 0) And xp > 0 And yp > 0
  Sleep 300   'Entprellzeit fuer die Maus
  amax = Pmap(xp + 1, 2)
  bmin = Pmap(yp + 1, 3)
  Draw String (amax - 0.5 * TextWidth("+"), bmin + 0.5 * TextHeight("+")), "+"
  
  Print "Rechte untere Ecke des gewuenschten Ausschnittes anklicken!"
  Do
    Sleep 1
    Getmouse xp, yp,,Taste
  Loop Until Bit(Taste, 0) And xp > 0 And yp > 0
  Sleep 300
  amin = Pmap(xp + 1, 2)
  bmax = Pmap(yp + 1, 3)
  Draw String (amin - 0.5 * TextWidth("+"), bmax + 0.5 * TextHeight("+")), "+"
  
  Line (amin, bmax) - (amax, bmin),,B
  Sleep 1000  

End Sub
      

'----------------------------- Hauptprogramm: ---------------------------------

Info

Do
  amin = -2.5
  amax = 1
  bmin = -1.5
  bmax = 1.5
  Do
    Mandel
    Print "F1: Zoom   F2: Gesamtbild    ESC: Ende"
    Do
      key = Getkey
    Loop Until key = SC_F1 Or key = SC_F2 Or key = SC_ESC
    If key = SC_F1 Then Zoom
  Loop Until key = SC_F2 Or key = SC_ESC
Loop Until key = SC_ESC

End
 