'==============================================================================
' Newton. bas
' Newton-Fraktal mit Zoom-Funktion
' Erstellt am 28.12.10
' Letzte Bearbeitung am 20.12.13
'==============================================================================


#INCLUDE "Inc\KOMPLEX.bas"
#INCLUDE "Inc\GRAPHIK.bas"
#INCLUDE "Inc\TUI.bas"


Dim Shared As Double xmin, xmax, ymin, ymax
Dim Shared As Integer k, key, n = 3
Dim Shared As COMPLEX z0(5)


Sub Info
  
  Color 0, 7
  Screen 0
  
  Zentriert(1, "Newton-Fraktal (siehe auch Wikipedia)")
  Print
  Print "Fuer die Gleichung z^n = 1, also die n-te Einheitswurzel, lautet das"
  Print "Newton-Verfahren zur iterativen Loesung:"
  Print
  Print "  z -> z - (z^n - 1)/(n * z^(n-1)) = (1 - 1/n)*z + 1/(n*z^(n-1))"
  Print "  mit z = x + i*y."
  Print 
  Print "Es wird fuer jedes Wertepaar x und y innerhalb der x-y-Ebene untersucht,"
  Print "wie schnell die aus der Iteration erzeugte Folge der z-Werte gegen eine"
  Print "der n Nullstellen konvergiert, wobei nach maximal 100 Iterationen"
  Print "abgebrochen wird. Die Zahl der Iterationen wird in der x-y-Ebene fuer"
  Print "jeden Wert von (x, y) als Helligkeitswert eingetragen. Der Farbwert"
  Print "entspricht der Nullstelle, gegen die z konvergiert."
  Print
  Print "In der berechneten Grafik kann mit Hilfe der Maus ein Ausschnitt zum"
  Print "Vergroessern ausgewaehlt werden (Zoom-Funktion)."
  Print
  Print "Siehe auch Wikipedia 'Newton-Fraktal'"
  Print
  Print "n (ganzzahlig, max. 6): ";
  n = Val(Edit(Str(n), 10))
  Locate 25, 1
  
End Sub


Sub Newton
  
  Dim As Single x, y, dx, dy
  Dim As COMPLEX z, a, b
  Dim As Integer i, j, code, hell, count
  
  InitGfx("Newton-Fraktal")
  View Print 29 To 30
  View(0, 0) - (640, 440)
  Window (xmin, ymin)-(xmax, ymax)
  
  dx = (xmax - xmin) / 640
  dy = (ymax - ymin) / 480
  a.Re = 1 - 1/n
  a. Im = 0
  b. Re = 1/n
  b.Im = 0
  
  For i = 1 To 640
    x = xmin + i * dx
    For j = 1 To 480
      y = ymin + j * dy
      z.Re = x
      z.Im = y
      count = 0
      Do
        z = CAdd(CMult(a, z), CDiv(b, CPotenz(z, n-1)))
        count = count + 1
        For k = 0 To n - 1
          If CAbs(CSub(z, z0(k))) < 1e-5 Then
            code = k + 1
            Exit Do
          End If
        Next k
      Loop Until count > 100
      If count > 100 Then code = 0
      hell = 255 - 25 * Sqr(count)
      Select Case code
      Case 0
        Pset (x, y), Rgb(hell, hell, hell)
      Case 1
        Pset (x, y), Rgb(hell, 0, 0)
      Case 2
        Pset (x, y), Rgb(hell, hell, 0)
      Case 3
        Pset (x, y), Rgb(0, hell, 0)
      Case 4
        Pset (x, y), Rgb(0, hell, hell)
      Case 5
        Pset (x, y), Rgb(0, 0, hell)
      Case 6
        Pset (x, y), Rgb(hell, 0, hell)
      End Select
    Next
  Next
  
End Sub


Sub Zoom
  
  Dim As Integer xp, yp, Taste
  
  Print "Linke obere Ecke des gewuenschten Ausschnittes anklicken!"
  Do
    Sleep 1
    Getmouse xp, yp,,Taste
  Loop Until Bit(Taste, 0) And xp > 0 And yp > 0
  Sleep 300 'Entprellzeit fuer die Maus  
  xmin = Pmap(xp + 1, 2)
  ymax = Pmap(yp + 1, 3)
  Draw String (xmin - 0.5 * TextWidth("+"), ymax + 0.5 * TextHeight("+")), "+" 
  
  Print "Rechte untere Ecke des gewuenschten Ausschnittes anklicken!"
  Do
    Sleep 1
    Getmouse xp, yp,,Taste
  Loop Until Bit(Taste, 0) And xp > 0 And yp > 0
  Sleep 300
  xmax = Pmap(xp + 1, 2)
  ymin = Pmap(yp + 1, 3)
  Draw String (xmax - 0.5 * TextWidth("+"), ymin + 0.5 * TextHeight("+")), "+"
  
  Line (xmax, ymin) - (xmin, ymax),,B
  Sleep 1000  
  
End Sub


'---------------------------- Hauptprogramm: ----------------------------------

Do
  
  xmin = -2
  xmax = 2
  ymin = -1.5
  ymax = 1.5
  Info
  'Nullstellen von z^n:
  For k = 0 To n - 1
    z0(k) = CTransform(1, 2 * pi * k / n)
  Next k
  
  Do
    Newton
    print "F1: Zoom   F2: Zum Anfang   ESC: Ende"
    Do
      key = Getkey
    Loop Until key = SC_F1 Or key = SC_F2 Or key = SC_ESC
    If key = SC_F1 Then Zoom
  Loop Until key = SC_F2 Or key = SC_ESC
  
Loop Until key = SC_ESC

End


